@@ -45,7 +45,7 @@ from paste.util import import_string
 
 import evalcontext
 from weberror import errormiddleware, formatter, collector
-from weberror.util import escaping
+from weberror.util import security
 from tempita import HTMLTemplate
 from webob import Request, Response
 from webob import exc
@@ -163,7 +163,25 @@ def get_debug_info(func):
     debug_info_replacement.exposed = True
     return debug_info_replacement
 
+def check_csrf_token(func):
+    """
+    A decorator to verify that the sender is same-origin with the debug app
+    """
+    def new_fn(self, req):
+        if 'csrf_token' not in req.params:
+            return exc.HTTPForbidden("You must provide a CSRF token")
+
+        csrf_token = req.params['csrf_token']
+        if not security.valid_csrf_token(csrf_secret, csrf_token):
+            return exc.HTTPForbidden("Invalid CSRF token")
+
+        return func(self, req)
+
+    new_fn.exposed = True
+    return new_fn
+
 debug_counter = itertools.count(int(time.time()))
+csrf_secret = security.gen_csrf_secret()
 
 def get_debug_count(req):
     """
@@ -197,6 +215,7 @@ class EvalException(object):
                  xmlhttp_key=None, media_paths=None, 
                  templating_formatters=None, head_html='', footer_html='',
                  reporters=None, libraries=None,
+                 debug_url_prefix=None,
                  **params):
         self.libraries = libraries or []
         self.application = application
@@ -213,6 +232,12 @@ class EvalException(object):
             else:
                 xmlhttp_key = global_conf.get('xmlhttp_key', '_')
         self.xmlhttp_key = xmlhttp_key
+        if debug_url_prefix is None:
+            if global_conf is None:
+                debug_url_prefix = '_debug'
+            else:
+                debug_url_prefix = global_conf.get('debug_url_prefix', '_debug')
+        self.debug_url_prefix = debug_url_prefix.split('/')
         self.media_paths = media_paths or {}
         self.error_template = HTMLTemplate.from_filename(error_template_filename)
         if reporters is None:
@@ -229,13 +254,15 @@ class EvalException(object):
         environ['weberror.evalexception'] = environ['paste.evalexception'] = \
             self
         req = Request(environ)
-        if req.path_info_peek() == '_debug':
+        req_path = req.path_info.split('/')[1:len(self.debug_url_prefix) + 1]
+        if req_path == self.debug_url_prefix:
             return self.debug(req)(environ, start_response)
         else:
             return self.respond(environ, start_response)
 
     def debug(self, req):
-        assert req.path_info_pop() == '_debug'
+        for path_part in self.debug_url_prefix:
+            assert req.path_info_pop() == path_part
         next_part = req.path_info_pop()
         method = getattr(self, next_part, None)
         if method is None:
@@ -244,35 +271,6 @@ class EvalException(object):
             return exc.HTTPForbidden('Access to %r is forbidden' % next_part)
         return method(req)
     
-    def relay(self, req):
-        """Relay a request to a remote machine for JS proxying"""
-        host = req.GET['host']
-        conn = httplib.HTTPConnection(host)
-        headers = req.headers
-        
-        # Re-assemble the query string
-        query_str = {}
-        for param, val in req.GET.iteritems():
-            if param in ['host', 'path']: continue
-            query_str[param] = val
-        query_str = urllib.urlencode(query_str)
-        
-        # Transport a GET or a POST
-        if req.method == 'GET':
-            conn.request("GET", '%s?%s' % (req.GET['path'], query_str), headers=headers)
-        elif req.method == 'POST':
-            conn.request("POST", req.GET['path'], req.body, headers=headers)
-        
-        # Handle the response and pull out the headers to proxy back
-        resp = conn.getresponse()
-        res = Response()
-        for header, value in resp.getheaders():
-            if header.lower() in ['server', 'date']: continue
-            res.headers[header] = value
-        res.body = resp.read()
-        return res
-    relay.exposed=True
-    
     def post_traceback(self, req):
         """Post the long XML traceback to the host and path provided"""
         debug_info = req.debug_info
@@ -292,7 +290,7 @@ class EvalException(object):
             res.headers[header] = value
         res.body = resp.read()
         return res
-    post_traceback = get_debug_info(post_traceback)
+    post_traceback = check_csrf_token(get_debug_info(post_traceback))
     
     def media(self, req):
         """Static path where images and other files live"""
@@ -379,7 +377,7 @@ class EvalException(object):
                preserve_whitespace(output)))
         return res
 
-    exec_input = get_debug_info(exec_input)
+    exec_input = check_csrf_token(get_debug_info(exec_input))
 
     def source_code(self, req):
         location = req.params['location']
@@ -388,7 +386,7 @@ class EvalException(object):
         if module is None:
             # Something weird indeed
             res = Response(content_type='text/html', charset='utf8')
-            res.body = 'The module <code>%s</code> does not have an entry in sys.modules' % module_name
+            res.unicode_body = 'The module <code>%s</code> does not have an entry in sys.modules' % html_quote(module_name)
             return res
         filename = module.__file__
         if filename[-4:] in ('.pyc', '.pyo'):
@@ -401,7 +399,7 @@ class EvalException(object):
         html = (
             ('<div>Module: <b>%s</b> file: %s</div>'
              '<style type="text/css">%s</style>'
-             % (module_name, filename, formatter.pygments_css))
+             % (html_quote(module_name), html_quote(filename), formatter.pygments_css))
             + formatter.highlight(filename, source, linenos=True))
         source_lines = len(source.splitlines())
         if source_lines < 60:
@@ -416,7 +414,7 @@ class EvalException(object):
         req = Request(environ)
         if req.environ.get('paste.throw_errors'):
             return self.application(environ, start_response)
-        base_path = req.application_url + '/_debug'
+        base_path = req.application_url + '/' + '/'.join(self.debug_url_prefix)
         req.environ['paste.throw_errors'] = True
         started = []
         def detect_start_response(status, headers, exc_info=None):
@@ -575,6 +573,7 @@ class DebugInfo(object):
             template_data=template_data,
             set_tab=tab,
             prefix=self.base_path,
+            csrf_token=security.generate_csrf_token(csrf_secret),
             counter=self.counter,
             )
         return [page]
